@@ -22,13 +22,38 @@ class Tk::BWidget::SpinBox
   WidgetClassName = 'SpinBox'.freeze
   WidgetClassNames[WidgetClassName] = self
 
-  def entrybind(*args)
-    _bind([path, 'bind'], *args)
+  def __tkvariable_optkeys
+    super() << 'helpvar'
+  end
+  private :__tkvariable_optkeys
+
+  #def entrybind(*args)
+  #  _bind([path, 'bind'], *args)
+  #  self
+  #end
+  def entrybind(context, *args)
+    #if args[0].kind_of?(Proc) || args[0].kind_of?(Method)
+    if TkComm._callback_entry?(args[0]) || !block_given?
+      cmd = args.shift
+    else
+      cmd = Proc.new
+    end
+    _bind([path, 'bind'], context, cmd, *args)
     self
   end
 
-  def entrybind_append(*args)
-    _bind_append([path, 'bind'], *args)
+  #def entrybind_append(*args)
+  #  _bind_append([path, 'bind'], *args)
+  #  self
+  #end
+  def entrybind_append(context, *args)
+    #if args[0].kind_of?(Proc) || args[0].kind_of?(Method)
+    if TkComm._callback_entry?(args[0]) || !block_given?
+      cmd = args.shift
+    else
+      cmd = Proc.new
+    end
+    _bind_append([path, 'bind'], context, cmd, *args)
     self
   end
 
