@@ -17,7 +17,13 @@ module Tk::Tile::Style
 end
 
 class << Tk::Tile::Style
-  def default(style, keys=nil)
+  def default(style=nil, keys=nil)
+    if style.kind_of?(Hash)
+      keys = style
+      style = nil
+    end
+    style = '.' unless style
+
     if keys && keys != None
       tk_call('style', 'default', style, *hash_kv(keys))
     else
@@ -25,7 +31,13 @@ class << Tk::Tile::Style
     end
   end
 
-  def map(style, keys=nil)
+  def map(style=nil, keys=nil)
+    if style.kind_of?(Hash)
+      keys = style
+      style = nil
+    end
+    style = '.' unless style
+
     if keys && keys != None
       tk_call('style', 'map', style, *hash_kv(keys))
     else
@@ -33,7 +45,13 @@ class << Tk::Tile::Style
     end
   end
 
-  def layout(style, spec=nil)
+  def layout(style=nil, spec=nil)
+    if style.kind_of?(Hash)
+      spec = style
+      style = nil
+    end
+    style = '.' unless style
+
     if spec
       tk_call('style', 'layout', style, spec)
     else
@@ -49,15 +67,21 @@ class << Tk::Tile::Style
     list(tk_call('style', 'element', 'names'))
   end
 
+  def element_options(elem)
+    simplelist(tk_call('style', 'element', 'options', elem))
+  end
+
   def theme_create(name, keys=nil)
     if keys && keys != None
-      tk_call('style', 'theme', 'create', name, type, *hash_kv(keys))
+      tk_call('style', 'theme', 'create', name, *hash_kv(keys))
     else
-      tk_call('style', 'theme', 'create', name, type)
+      tk_call('style', 'theme', 'create', name)
     end
   end
 
   def theme_settings(name, cmd=nil, &b)
+    cmd = Proc.new(&b) if !cmd && b
+    tk_call('style', 'theme', 'settings', name, cmd)
   end
 
   def theme_names()
@@ -65,6 +89,6 @@ class << Tk::Tile::Style
   end
 
   def theme_use(name)
-    tk_call('style', 'use', name)
+    tk_call('style', 'theme', 'use', name)
   end
 end
