@@ -2,8 +2,8 @@
 
   file.c -
 
-  $Author: nobu $
-  $Date: 2004/10/29 12:28:31 $
+  $Author: akr $
+  $Date: 2005/09/18 18:22:39 $
   created at: Mon Nov 15 12:24:34 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -64,8 +64,8 @@ char *strrchr _((const char*,const char));
 #include <sys/mkdev.h>
 #endif
 
-#ifndef HAVE_LSTAT
-#define lstat(path,st) stat(path,st)
+#if !defined HAVE_LSTAT && !defined lstat
+#define lstat stat
 #endif
 #if !HAVE_FSEEKO && !defined(fseeko)
 # define fseeko  fseek
@@ -96,7 +96,7 @@ apply2files(func, vargs, arg)
 
 /*
  *  call-seq:
- *     file.path -> file_ame
+ *     file.path -> filename
  *  
  *  Returns the pathname used to create <i>file</i> as a string. Does
  *  not normalize the name.
@@ -1744,15 +1744,18 @@ rb_file_chown(obj, owner, group)
     VALUE obj, owner, group;
 {
     OpenFile *fptr;
+    int o, g;
 
     rb_secure(2);
+    o = NUM2INT(owner);
+    g = NUM2INT(group);
     GetOpenFile(obj, fptr);
 #if defined(DJGPP) || defined(__CYGWIN32__) || defined(_WIN32) || defined(__EMX__)
     if (!fptr->path) return Qnil;
-    if (chown(fptr->path, NUM2INT(owner), NUM2INT(group)) == -1)
+    if (chown(fptr->path, o, g) == -1)
 	rb_sys_fail(fptr->path);
 #else
-    if (fchown(fileno(fptr->f), NUM2INT(owner), NUM2INT(group)) == -1)
+    if (fchown(fileno(fptr->f), o, g) == -1)
 	rb_sys_fail(fptr->path);
 #endif
 
@@ -2672,7 +2675,7 @@ rb_file_s_extname(klass, fname)
  	p++;
  
      e = strrchr(p, '.');	/* get the last dot of the last component */
-     if (!e || e == p)		/* no dot, or the only dot is first? */
+     if (!e || e == p || !e[1])	/* no dot, or the only dot is first or end? */
 	 return rb_str_new2("");
      extname = rb_str_new(e, chompdirsep(e) - e);	/* keep the dot, too! */
      OBJ_INFECT(extname, fname);
@@ -2717,7 +2720,7 @@ rb_file_join(ary, sep)
     long len, i;
     int taint = 0;
     VALUE result, tmp;
-    char *name;
+    char *name, *tail;
 
     if (RARRAY(ary)->len == 0) return rb_str_new(0, 0);
     if (OBJ_TAINTED(ary)) taint = 1;
@@ -2754,11 +2757,18 @@ rb_file_join(ary, sep)
 	    }
 	    break;
 	  default:
-	    tmp = rb_obj_as_string(tmp);
+	    StringValueCStr(tmp);
 	}
 	name = StringValueCStr(result);
-	if (i > 0 && !NIL_P(sep) && !*chompdirsep(name))
-	    rb_str_buf_append(result, sep);
+	if (i > 0 && !NIL_P(sep)) {
+	    tail = chompdirsep(name);
+	    if (isdirsep(RSTRING(tmp)->ptr[0])) {
+		RSTRING(result)->len = tail - name;
+	    }
+	    else if (!*tail) {
+		rb_str_buf_append(result, sep);
+	    }
+	}
 	rb_str_buf_append(result, tmp);
 	if (OBJ_TAINTED(tmp)) taint = 1;
     }
@@ -2804,11 +2814,14 @@ static VALUE
 rb_file_s_truncate(klass, path, len)
     VALUE klass, path, len;
 {
+    off_t pos;
+
     rb_secure(2);
+    pos = NUM2OFFT(len);
     SafeStringValue(path);
 
 #ifdef HAVE_TRUNCATE
-    if (truncate(StringValueCStr(path), NUM2OFFT(len)) < 0)
+    if (truncate(StringValueCStr(path), pos) < 0)
 	rb_sys_fail(RSTRING(path)->ptr);
 #else
 # ifdef HAVE_CHSIZE
@@ -2824,7 +2837,7 @@ rb_file_s_truncate(klass, path, len)
 	    rb_sys_fail(RSTRING(path)->ptr);
 	}
 #  endif
-	if (chsize(tmpfd, NUM2OFFT(len)) < 0) {
+	if (chsize(tmpfd, pos) < 0) {
 	    close(tmpfd);
 	    rb_sys_fail(RSTRING(path)->ptr);
 	}
@@ -2857,8 +2870,10 @@ rb_file_truncate(obj, len)
 {
     OpenFile *fptr;
     FILE *f;
+    off_t pos;
 
     rb_secure(2);
+    pos = NUM2OFFT(len);
     GetOpenFile(obj, fptr);
     if (!(fptr->mode & FMODE_WRITABLE)) {
 	rb_raise(rb_eIOError, "not opened for writing");
@@ -2867,11 +2882,11 @@ rb_file_truncate(obj, len)
     fflush(f);
     fseeko(f, (off_t)0, SEEK_CUR);
 #ifdef HAVE_TRUNCATE
-    if (ftruncate(fileno(f), NUM2OFFT(len)) < 0)
+    if (ftruncate(fileno(f), pos) < 0)
 	rb_sys_fail(fptr->path);
 #else
 # ifdef HAVE_CHSIZE
-    if (chsize(fileno(f), NUM2OFFT(len)) < 0)
+    if (chsize(fileno(f), pos) < 0)
 	rb_sys_fail(fptr->path);
 # else
     rb_notimplement();
@@ -2893,14 +2908,18 @@ rb_file_truncate(obj, len)
 #  define LOCK_UN 8
 # endif
 
-#if 0
+#if 1
 static int
 rb_thread_flock(fd, op, fptr)
     int fd, op;
     OpenFile *fptr;
 {
     if (rb_thread_alone() || (op & LOCK_NB)) {
-	return flock(fd, op);
+        int ret;
+        TRAP_BEG;
+	ret = flock(fd, op);
+        TRAP_END;
+	return ret;
     }
     op |= LOCK_NB;
     while (flock(fd, op) < 0) {
@@ -2958,15 +2977,17 @@ rb_file_flock(obj, operation)
 {
 #ifndef __CHECKER__
     OpenFile *fptr;
+    int op;
 
     rb_secure(2);
+    op = NUM2INT(operation);
     GetOpenFile(obj, fptr);
 
     if (fptr->mode & FMODE_WRITABLE) {
 	fflush(GetWriteFile(fptr));
     }
   retry:
-    if (flock(fileno(fptr->f), NUM2INT(operation)) < 0) {
+    if (flock(fileno(fptr->f), op) < 0) {
         switch (errno) {
           case EAGAIN:
           case EACCES:
