@@ -12,6 +12,7 @@
 
 #include "ruby.h"
 #include "rubysig.h"
+#include "dln.h"
 #include <fcntl.h>
 #include <process.h>
 #include <sys/stat.h>
@@ -88,12 +89,11 @@
 
 bool NtSyncProcess = TRUE;
 
-static struct ChildRecord *CreateChild(char *, char *, SECURITY_ATTRIBUTES *, HANDLE, HANDLE, HANDLE);
-static int make_cmdvector(const char *, char ***);
+static struct ChildRecord *CreateChild(const char *, const char *, SECURITY_ATTRIBUTES *, HANDLE, HANDLE, HANDLE);
 static bool has_redirection(const char *);
 static void StartSockets ();
 static DWORD wait_events(HANDLE event, DWORD timeout);
-#if !defined(__BORLANDC__) && !defined(_WIN32_WCE)
+#if !defined(_WIN32_WCE)
 static int rb_w32_open_osfhandle(long osfhandle, int flags);
 #else
 #define rb_w32_open_osfhandle(osfhandle, flags) _open_osfhandle(osfhandle, flags)
@@ -157,6 +157,7 @@ static struct {
     {	ERROR_NEGATIVE_SEEK,		EINVAL		},
     {	ERROR_SEEK_ON_DEVICE,		EACCES		},
     {	ERROR_DIR_NOT_EMPTY,		ENOTEMPTY	},
+    {	ERROR_DIRECTORY,		ENOTDIR		},
     {	ERROR_NOT_LOCKED,		EACCES		},
     {	ERROR_BAD_PATHNAME,		ENOENT		},
     {	ERROR_MAX_THRDS_REACHED,	EAGAIN		},
@@ -247,7 +248,8 @@ static struct {
 /* interrupt stuff */
 static HANDLE interrupted_event;
 
-HANDLE GetCurrentThreadHandle(void)
+HANDLE
+GetCurrentThreadHandle(void)
 {
     static HANDLE current_process_handle = NULL;
     HANDLE h;
@@ -293,6 +295,7 @@ flock_winnt(VALUE self, int argc, VALUE* argv)
 			  0, LK_LEN, LK_LEN, &o), i);
 	break;
       case LOCK_UN:		/* unlock lock */
+      case LOCK_UN|LOCK_NB:	/* unlock is always non-blocking, I hope */
 	LK_ERR(UnlockFileEx(fh, 0, LK_LEN, LK_LEN, &o), i);
 	break;
       default:            /* unknown */
@@ -320,6 +323,7 @@ flock_win95(VALUE self, int argc, VALUE* argv)
 	LK_ERR(LockFile(fh, 0, 0, LK_LEN, LK_LEN), i);
 	break;
       case LOCK_UN:
+      case LOCK_UN|LOCK_NB:
 	LK_ERR(UnlockFile(fh, 0, 0, LK_LEN, LK_LEN), i);
 	break;
       default:
@@ -363,14 +367,10 @@ NtInitialize(int *argc, char ***argv)
     WORD version;
     int ret;
 
-#ifdef __BORLANDC__
-    _controlfp(0x5, 0x5);
-#endif
-
     //
     // subvert cmd.exe's feeble attempt at command line parsing
     //
-    *argc = make_cmdvector(GetCommandLine(), argv);
+    *argc = rb_w32_cmdvector(GetCommandLine(), argv);
 
     //
     // Now set up the correct time stuff
@@ -387,7 +387,8 @@ NtInitialize(int *argc, char ***argv)
 #endif
 }
 
-char *getlogin()
+char *
+getlogin()
 {
     char buffer[200];
     DWORD len = 200;
@@ -410,7 +411,7 @@ char *getlogin()
 
 static struct ChildRecord {
     HANDLE hProcess;	/* process handle */
-    pid_t pid;		/* process id */
+    rb_pid_t pid;	/* process id */
 } ChildRecord[MAXCHILDNUM];
 
 #define FOREACH_CHILD(v) do { \
@@ -428,7 +429,7 @@ FindFirstChildSlot(void)
 }
 
 static struct ChildRecord *
-FindChildSlot(pid_t pid)
+FindChildSlot(rb_pid_t pid)
 {
 
     FOREACH_CHILD(child) {
@@ -462,7 +463,8 @@ FindFreeChildSlot(void)
 }
 
 
-int SafeFree(char **vec, int vecc)
+int
+SafeFree(char **vec, int vecc)
 {
     //   vec
     //   |
@@ -590,7 +592,7 @@ rb_w32_get_osfhandle(int fh)
     return _get_osfhandle(fh);
 }
 
-pid_t
+rb_pid_t
 pipe_exec(char *cmd, int mode, FILE **fpr, FILE **fpw)
 {
     struct ChildRecord* child;
@@ -730,9 +732,7 @@ pipe_exec(char *cmd, int mode, FILE **fpr, FILE **fpw)
 extern VALUE rb_last_status;
 
 int
-do_spawn(mode, cmd)
-int mode;
-char *cmd;
+do_spawn(int mode, char *cmd)
 {
     struct ChildRecord *child;
     DWORD exitcode;
@@ -769,10 +769,7 @@ char *cmd;
 }
 
 int
-do_aspawn(mode, prog, argv)
-int mode;
-char *prog;
-char **argv;
+do_aspawn(int mode, char *prog, char **argv)
 {
     char *cmd, *p, *q, *s, **t;
     int len, n, bs, quote;
@@ -863,15 +860,17 @@ char **argv;
 }
 
 static struct ChildRecord *
-CreateChild(char *cmd, char *prog, SECURITY_ATTRIBUTES *psa, HANDLE hInput, HANDLE hOutput, HANDLE hError)
+CreateChild(const char *cmd, const char *prog, SECURITY_ATTRIBUTES *psa,
+	    HANDLE hInput, HANDLE hOutput, HANDLE hError)
 {
     BOOL fRet;
     DWORD  dwCreationFlags;
     STARTUPINFO aStartupInfo;
     PROCESS_INFORMATION aProcessInformation;
     SECURITY_ATTRIBUTES sa;
-    char *shell;
+    const char *shell;
     struct ChildRecord *child;
+    char *p = NULL;
 
     if (!cmd && !prog) {
 	errno = EFAULT;
@@ -919,7 +918,9 @@ CreateChild(char *cmd, char *prog, SECURITY_ATTRIBUTES *psa, HANDLE hInput, HAND
     dwCreationFlags = (NORMAL_PRIORITY_CLASS);
 
     if (prog) {
-	shell = prog;
+	if (!(p = dln_find_exe(prog, NULL))) {
+	    shell = prog;
+	}
     }
     else {
 	int redir = -1;
@@ -929,7 +930,7 @@ CreateChild(char *cmd, char *prog, SECURITY_ATTRIBUTES *psa, HANDLE hInput, HAND
 	    if (ISSPACE(*prog)) {
 		len = prog - cmd;
 		do ++prog; while (ISSPACE(*prog));
-		if (!*prog) break;
+		if (!*prog--) break;
 	    }
 	    else {
 		len = 0;
@@ -949,15 +950,45 @@ CreateChild(char *cmd, char *prog, SECURITY_ATTRIBUTES *psa, HANDLE hInput, HAND
 	    cmd = tmp;
 	}
 	else {
-	    char *tmp = ALLOCA_N(char, len + 1);
-	    sprintf(tmp, "%.*s", len, cmd);
-	    cmd = tmp;
 	    shell = NULL;
+	    prog = cmd;
+	    for (;;) {
+		if (!*prog) {
+		    p = dln_find_exe(cmd, NULL);
+		    break;
+		}
+		if (strchr(".:*?\"/\\", *prog)) {
+		    if (cmd[len]) {
+			char *tmp = ALLOCA_N(char, len + 1);
+			memcpy(tmp, cmd, len);
+			tmp[len] = 0;
+			cmd = tmp;
+		    }
+		    break;
+		}
+		if (ISSPACE(*prog) || strchr("<>|", *prog)) {
+		    len = prog - cmd;
+		    p = ALLOCA_N(char, len + 1);
+		    memcpy(p, cmd, len);
+		    p[len] = 0;
+		    p = dln_find_exe(p, NULL);
+		    break;
+		}
+		prog++;
+	    }
+	}
+    }
+    if (p) {
+	shell = p;
+	while (*p) {
+	    if ((unsigned char)*p == '/')
+		*p = '\\';
+	    p = CharNext(p);
 	}
     }
 
     RUBY_CRITICAL({
-	fRet = CreateProcess(shell, cmd, psa, psa,
+	fRet = CreateProcess(shell, (char *)cmd, psa, psa,
 			     psa->bInheritHandle, dwCreationFlags, NULL, NULL,
 			     &aStartupInfo, &aProcessInformation);
 	errno = map_errno(GetLastError());
@@ -971,7 +1002,7 @@ CreateChild(char *cmd, char *prog, SECURITY_ATTRIBUTES *psa, HANDLE hInput, HAND
     CloseHandle(aProcessInformation.hThread);
 
     child->hProcess = aProcessInformation.hProcess;
-    child->pid = (pid_t)aProcessInformation.dwProcessId;
+    child->pid = (rb_pid_t)aProcessInformation.dwProcessId;
 
     if (!IsWinNT()) {
 	/* On Win9x, make pid positive similarly to cygwin and perl */
@@ -996,7 +1027,7 @@ typedef struct _NtCmdLineElement {
 #define NTMALLOC 0x2	// string in element was malloc'ed
 #define NTSTRING 0x4	// element contains a quoted string
 
-static void
+static int
 insert(const char *path, VALUE vinfo)
 {
     NtCmdLineElement *tmpcurr;
@@ -1010,6 +1041,8 @@ insert(const char *path, VALUE vinfo)
     strcpy(tmpcurr->str, path);
     **tail = tmpcurr;
     *tail = &tmpcurr->next;
+
+    return 0;
 }
 
 #ifdef HAVE_SYS_PARAM_H
@@ -1034,7 +1067,7 @@ cmdglob(NtCmdLineElement *patt, NtCmdLineElement **tail)
     for (p = buf; *p; p = CharNext(p))
 	if (*p == '\\')
 	    *p = '/';
-    rb_globi(buf, insert, (VALUE)&tail);
+    ruby_globi(buf, 0, insert, (VALUE)&tail);
     if (buf != buffer)
 	free(buf);
 
@@ -1098,8 +1131,8 @@ skipspace(char *ptr)
     return ptr;
 }
 
-static int 
-make_cmdvector(const char *cmd, char ***vec)
+int 
+rb_w32_cmdvector(const char *cmd, char ***vec)
 {
     int cmdlen, globbing, len, i;
     int elements, strsz, done;
@@ -1613,6 +1646,19 @@ rb_w32_open_osfhandle(long osfhandle, int flags)
 
 #endif
 
+#ifdef __BORLANDC__
+static int
+rb_w32_open_osfhandle(long osfhandle, int flags)
+{
+    int fd = _open_osfhandle(osfhandle, flags);
+    if (fd == -1) {
+	errno = EMFILE;		/* too many open files */
+	_doserrno = 0L;		/* not an OS error */
+    }
+    return fd;
+}
+#endif
+
 #undef getsockopt
 
 static int
@@ -1659,6 +1705,15 @@ rb_w32_strerror(int e)
     DWORD source = 0;
     char *p;
 
+#if defined __BORLANDC__ && defined ENOTEMPTY // _sys_errlist is broken
+    switch (e) {
+      case ENAMETOOLONG:
+	return "Filename too long";
+      case ENOTEMPTY:
+	return "Directory not empty";
+    }
+#endif
+
     if (e < 0 || e > sys_nerr) {
 	if (e < 0)
 	    e = GetLastError();
@@ -1667,13 +1722,17 @@ rb_w32_strerror(int e)
 			  buffer, 512, NULL) == 0) {
 	    strcpy(buffer, "Unknown Error");
 	}
-	for (p = buffer + strlen(buffer) - 1; buffer <= p; p--) {
-	    if (*p != '\r' && *p != '\n') break;
-	    *p = 0;
-	}
-	return buffer;
     }
-    return strerror(e);
+    else {
+	strncpy(buffer, strerror(e), sizeof(buffer));
+	buffer[sizeof(buffer) - 1] = 0;
+    }
+
+    for (p = buffer + strlen(buffer) - 1; buffer <= p; p--) {
+	if (*p != '\r' && *p != '\n') break;
+	*p = 0;
+    }
+    return buffer;
 }
 
 //
@@ -1690,38 +1749,38 @@ rb_w32_strerror(int e)
 #define ROOT_UID	0
 #define ROOT_GID	0
 
-UIDTYPE
+rb_uid_t
 getuid(void)
 {
 	return ROOT_UID;
 }
 
-UIDTYPE
+rb_uid_t
 geteuid(void)
 {
 	return ROOT_UID;
 }
 
-GIDTYPE
+rb_gid_t
 getgid(void)
 {
 	return ROOT_GID;
 }
 
-GIDTYPE
+rb_gid_t
 getegid(void)
 {
     return ROOT_GID;
 }
 
 int
-setuid(int uid)
+setuid(rb_uid_t uid)
 { 
     return (uid == ROOT_UID ? 0 : -1);
 }
 
 int
-setgid(int gid)
+setgid(rb_gid_t gid)
 {
     return (gid == ROOT_GID ? 0 : -1);
 }
@@ -1787,7 +1846,12 @@ rb_w32_fdclr(int fd, fd_set *set)
 int
 rb_w32_fdisset(int fd, fd_set *set)
 {
-       return __WSAFDIsSet(TO_SOCKET(fd), set);
+    int ret;
+    SOCKET s = TO_SOCKET(fd);
+    if (s == (SOCKET)INVALID_HANDLE_VALUE)
+        return 0;
+    RUBY_CRITICAL(ret = __WSAFDIsSet(s, set));
+    return ret;
 }
 
 //
@@ -1984,8 +2048,16 @@ rb_w32_connect(int s, struct sockaddr *addr, int addrlen)
     }
     RUBY_CRITICAL({
 	r = connect(TO_SOCKET(s), addr, addrlen);
-	if (r == SOCKET_ERROR)
-	    errno = map_errno(WSAGetLastError());
+	if (r == SOCKET_ERROR) {
+	    r = WSAGetLastError();
+	    if (r != WSAEWOULDBLOCK) {
+		errno = map_errno(r);
+	    }
+	    else {
+		errno = EINPROGRESS;
+		r = -1;
+	    }
+	}
     });
     return r;
 }
@@ -2332,9 +2404,9 @@ void endservent() {}
 
 struct netent *getnetent (void) {return (struct netent *) NULL;}
 
-struct netent *getnetbyaddr(char *name) {return (struct netent *)NULL;}
+struct netent *getnetbyaddr(long net, int type) {return (struct netent *)NULL;}
 
-struct netent *getnetbyname(long net, int type) {return (struct netent *)NULL;}
+struct netent *getnetbyname(char *name) {return (struct netent *)NULL;}
 
 struct protoent *getprotoent (void) {return (struct protoent *) NULL;}
 
@@ -2348,11 +2420,48 @@ void setprotoent (int stayopen) {}
 
 void setservent (int stayopen) {}
 
+int
+fcntl(int fd, int cmd, ...)
+{
+    SOCKET sock = TO_SOCKET(fd);
+    va_list va;
+    int arg;
+    int ret;
+    u_long ioctlArg;
+
+    if (!is_socket(sock)) {
+	errno = EBADF;
+	return -1;
+    }
+    if (cmd != F_SETFL) {
+	errno = EINVAL;
+	return -1;
+    }
+
+    va_start(va, cmd);
+    arg = va_arg(va, int);
+    va_end(va);
+    if (arg & O_NONBLOCK) {
+	ioctlArg = 1;
+    }
+    else {
+	ioctlArg = 0;
+    }
+    RUBY_CRITICAL({
+	ret = ioctlsocket(sock, FIONBIO, &ioctlArg);
+	if (ret == -1) {
+	    errno = map_errno(WSAGetLastError());
+	}
+    });
+
+    return ret;
+}
+
 #ifndef WNOHANG
 #define WNOHANG -1
 #endif
 
-static pid_t
+static rb_pid_t
 poll_child_status(struct ChildRecord *child, int *stat_loc)
 {
     DWORD exitcode;
@@ -2370,7 +2479,7 @@ poll_child_status(struct ChildRecord *child, int *stat_loc)
     }
     if (exitcode != STILL_ACTIVE) {
 	/* If already died, return immediatly. */
-	pid_t pid = child->pid;
+	rb_pid_t pid = child->pid;
 	CloseChildHandle(child);
 	if (stat_loc) *stat_loc = exitcode << 8;
 	return pid;
@@ -2378,8 +2487,8 @@ poll_child_status(struct ChildRecord *child, int *stat_loc)
     return 0;
 }
 
-pid_t
-waitpid (pid_t pid, int *stat_loc, int options)
+rb_pid_t
+waitpid (rb_pid_t pid, int *stat_loc, int options)
 {
     DWORD timeout;
 
@@ -2464,9 +2573,7 @@ gettimeofday(struct timeval *tv, struct timezone *tz)
 }
 
 char *
-rb_w32_getcwd(buffer, size)
-    char *buffer;
-    int size;
+rb_w32_getcwd(char *buffer, int size)
 {
     int length;
     char *bp;
@@ -2793,7 +2900,8 @@ rb_w32_times(struct tms *tmbuf)
 #define yield_once() Sleep(0)
 #define yield_until(condition) do yield_once(); while (!(condition))
 
-static DWORD wait_events(HANDLE event, DWORD timeout)
+static DWORD
+wait_events(HANDLE event, DWORD timeout)
 {
     HANDLE events[2];
     int count = 0;
@@ -2814,7 +2922,8 @@ static DWORD wait_events(HANDLE event, DWORD timeout)
     return ret;
 }
 
-static CRITICAL_SECTION* system_state(void)
+static CRITICAL_SECTION *
+system_state(void)
 {
     static int initialized = 0;
     static CRITICAL_SECTION syssect;
@@ -2829,7 +2938,8 @@ static CRITICAL_SECTION* system_state(void)
 static LONG flag_interrupt = -1;
 static volatile DWORD tlsi_interrupt = TLS_OUT_OF_INDEXES;
 
-void rb_w32_enter_critical(void)
+void
+rb_w32_enter_critical(void)
 {
     if (IsWinNT()) {
 	EnterCriticalSection(system_state());
@@ -2850,7 +2960,8 @@ void rb_w32_enter_critical(void)
     }
 }
 
-void rb_w32_leave_critical(void)
+void
+rb_w32_leave_critical(void)
 {
     if (IsWinNT()) {
 	LeaveCriticalSection(system_state());
@@ -2869,7 +2980,8 @@ struct handler_arg_t {
     HANDLE handshake;
 };
 
-static void rb_w32_call_handler(struct handler_arg_t* h)
+static void
+rb_w32_call_handler(struct handler_arg_t* h)
 {
     int status;
     RUBY_CRITICAL(rb_protect((VALUE (*)(VALUE))h->handler, (VALUE)h->arg, &h->status);
@@ -2882,10 +2994,9 @@ static void rb_w32_call_handler(struct handler_arg_t* h)
     yield_until(0);
 }
 
-static struct handler_arg_t* setup_handler(struct handler_arg_t *harg,
-					   int arg,
-					   void (*handler)(int),
-					   HANDLE handshake)
+static struct handler_arg_t *
+setup_handler(struct handler_arg_t *harg, int arg, void (*handler)(int),
+	      HANDLE handshake)
 {
     harg->handler = handler;
     harg->arg = arg;
@@ -2895,7 +3006,8 @@ static struct handler_arg_t* setup_handler(struct handler_arg_t *harg,
     return harg;
 }
 
-static void setup_call(CONTEXT* ctx, struct handler_arg_t *harg)
+static void
+setup_call(CONTEXT* ctx, struct handler_arg_t *harg)
 {
 #ifdef _M_IX86
     DWORD *esp = (DWORD *)ctx->Esp;
@@ -2910,7 +3022,8 @@ static void setup_call(CONTEXT* ctx, struct handler_arg_t *harg)
 #endif
 }
 
-int rb_w32_main_context(int arg, void (*handler)(int))
+int
+rb_w32_main_context(int arg, void (*handler)(int))
 {
     static HANDLE interrupt_done = NULL;
     struct handler_arg_t harg;
@@ -2971,7 +3084,8 @@ int rb_w32_main_context(int arg, void (*handler)(int))
     return TRUE;
 }
 
-int rb_w32_sleep(unsigned long msec)
+int
+rb_w32_sleep(unsigned long msec)
 {
     DWORD ret;
     RUBY_CRITICAL(ret = wait_events(NULL, msec));
@@ -2980,7 +3094,8 @@ int rb_w32_sleep(unsigned long msec)
     return ret != WAIT_TIMEOUT;
 }
 
-static void catch_interrupt(void)
+static void
+catch_interrupt(void)
 {
     yield_once();
     RUBY_CRITICAL(wait_events(NULL, 0));
@@ -2988,7 +3103,8 @@ static void catch_interrupt(void)
 }
 
 #undef fgetc
-int rb_w32_getc(FILE* stream)
+int
+rb_w32_getc(FILE* stream)
 {
     int c, trap_immediate = rb_trap_immediate;
 #ifndef _WIN32_WCE
@@ -3012,7 +3128,8 @@ int rb_w32_getc(FILE* stream)
 }
 
 #undef fputc
-int rb_w32_putc(int c, FILE* stream)
+int
+rb_w32_putc(int c, FILE* stream)
 {
     int trap_immediate = rb_trap_immediate;
 #ifndef _WIN32_WCE
@@ -3119,7 +3236,8 @@ rb_w32_asynchronize(asynchronous_func_t func, VALUE self,
     return val;
 }
 
-char **rb_w32_get_environ(void)
+char **
+rb_w32_get_environ(void)
 {
     char *envtop, *env;
     char **myenvtop, **myenv;
@@ -3152,7 +3270,8 @@ char **rb_w32_get_environ(void)
     return myenvtop;
 }
 
-void rb_w32_free_environ(char **env)
+void
+rb_w32_free_environ(char **env)
 {
     char **t = env;
 
@@ -3160,11 +3279,14 @@ void rb_w32_free_environ(char **env)
     free(env);
 }
 
-pid_t rb_w32_getpid(void)
+#undef getpid
+rb_pid_t
+rb_w32_getpid(void)
 {
-    pid_t pid;
+    rb_pid_t pid;
+
+    pid = getpid();
 
-    pid = _getpid();
     if (IsWin95()) pid = -pid;
 
     return pid;
@@ -3199,6 +3321,8 @@ rb_w32_close(int fd)
 	UnlockFile((HANDLE)sock, 0, 0, LK_LEN, LK_LEN);
 	return _close(fd);
     }
+    _set_osfhnd(fd, (SOCKET)INVALID_HANDLE_VALUE);
+    _close(fd);
     if (closesocket(sock) == SOCKET_ERROR) {
 	errno = map_errno(WSAGetLastError());
 	return -1;
@@ -3211,8 +3335,9 @@ unixtime_to_filetime(time_t time, FILETIME *ft)
 {
     struct tm *tm;
     SYSTEMTIME st;
+    FILETIME lt;
 
-    tm = gmtime(&time);
+    tm = localtime(&time);
     st.wYear = tm->tm_year + 1900;
     st.wMonth = tm->tm_mon + 1;
     st.wDayOfWeek = tm->tm_wday;
@@ -3221,17 +3346,14 @@ unixtime_to_filetime(time_t time, FILETIME *ft)
     st.wMinute = tm->tm_min;
     st.wSecond = tm->tm_sec;
     st.wMilliseconds = 0;
-    if (!SystemTimeToFileTime(&st, ft)) {
+    if (!SystemTimeToFileTime(&st, &lt) ||
+	!LocalFileTimeToFileTime(&lt, ft)) {
 	errno = map_errno(GetLastError());
 	return -1;
     }
     return 0;
 }
 
-#undef utime
-#ifdef __BORLANDC__
-#define utime _utime
-#endif
 int
 rb_w32_utime(const char *path, struct utimbuf *times)
 {
@@ -3292,6 +3414,80 @@ rb_w32_snprintf(char *buf, size_t size, const char *format, ...)
     return ret;
 }
 
+int
+rb_w32_mkdir(const char *path, int mode)
+{
+    int ret = -1;
+    RUBY_CRITICAL(do {
+	if (CreateDirectory(path, NULL) == FALSE) {
+	    errno = map_errno(GetLastError());
+	    break;
+	}
+	if (chmod(path, mode) == -1) {
+	    RemoveDirectory(path);
+	    break;
+	}
+	ret = 0;
+    } while (0));
+    return ret;
+}
+
+int
+rb_w32_rmdir(const char *path)
+{
+    int ret = 0;
+    RUBY_CRITICAL({
+	const DWORD attr = GetFileAttributes(path);
+	if (attr != (DWORD)-1 && (attr & FILE_ATTRIBUTE_READONLY)) {
+	    SetFileAttributes(path, attr & ~FILE_ATTRIBUTE_READONLY);
+	}
+	if (RemoveDirectory(path) == FALSE) {
+	    errno = map_errno(GetLastError());
+	    ret = -1;
+	    if (attr != (DWORD)-1 && (attr & FILE_ATTRIBUTE_READONLY)) {
+		SetFileAttributes(path, attr);
+	    }
+	}
+    });
+    return ret;
+}
+
+int
+rb_w32_unlink(const char *path)
+{
+    int ret = 0;
+    RUBY_CRITICAL({
+	const DWORD attr = GetFileAttributes(path);
+	if (attr != (DWORD)-1 && (attr & FILE_ATTRIBUTE_READONLY)) {
+	    SetFileAttributes(path, attr & ~FILE_ATTRIBUTE_READONLY);
+	}
+	if (DeleteFile(path) == FALSE) {
+	    errno = map_errno(GetLastError());
+	    ret = -1;
+	    if (attr != (DWORD)-1 && (attr & FILE_ATTRIBUTE_READONLY)) {
+		SetFileAttributes(path, attr);
+	    }
+	}
+    });
+    return ret;
+}
+
+#if !defined(__BORLANDC__) && !defined(_WIN32_WCE)
+int
+rb_w32_isatty(int fd)
+{
+    if (!(_osfile(fd) & FOPEN)) {
+	errno = EBADF;
+	return 0;
+    }
+    if (!(_osfile(fd) & FDEV)) {
+	errno = ENOTTY;
+	return 0;
+    }
+    return 1;
+}
+#endif
+
 //
 // Fix bcc32's stdio bug
 //
@@ -3320,9 +3516,9 @@ rb_w32_fopen(const char *path, const char *mode)
 }
 
 FILE *
-rb_w32_fdopen(int handle, char *type)
+rb_w32_fdopen(int handle, const char *type)
 {
-    FILE *f = (errno = 0, _fdopen(handle, type));
+    FILE *f = (errno = 0, _fdopen(handle, (char *)type));
     if (f == NULL && errno == 0) {
 	if (handle < 0)
 	    errno = EBADF;
